==> recharge_api-1.9.1-py3-none-any.whl: recharge/api/v1/products.py <==
from typing import Literal, TypedDict, Optional

from recharge.api import RechargeResource, RechargeScope, RechargeVersion

ProductDiscountType = Literal["percentage"]

ProductStorefrontPurchaseOptions = Literal[
    "subscription_only", "subscription_and_onetime"
]


class ProductCreateBodyOptional(TypedDict, total=False):
    charge_interval_frequency: int
    cutoff_day_of_month: int
    cutoff_day_of_week: int
    discount_amount: str
    discount_type: ProductDiscountType
    expire_after_specific_number_of_charges: str
    modifiable_properties: list[str]
    order_day_of_month: int
    order_day_of_week: int
    order_interval_frequency_options: list[int]
    storefront_purchase_options: ProductStorefrontPurchaseOptions


class ProductCreateBody(ProductCreateBodyOptional):
    shopify_product_id: int


class ProductImages(TypedDict, total=False):
    large: str
    medium: str
    original: str
    small: str


ProductOrderIntervalUnit = Literal["day", "week", "month"]


class ProductGetQuery(TypedDict, total=False):
    charge_interval_frequency: str
    created_at: str
    cutoff_day_of_month: str
    cutoff_day_of_week: str
    discount_amount: str
    discount_type: ProductDiscountType
    expire_after_specific_number_of_charges: str
    handle: str
    images: ProductImages
    modifiable_properties: list[str]
    number_charges_until_expiration: str
    order_day_of_month: str
    order_day_of_week: str
    order_interval_frequency: str
    order_interval_unit: ProductOrderIntervalUnit
    shopify_product_id: str
    storefront_purchase_options: ProductStorefrontPurchaseOptions
    title: str
    updated_at: str


class ProductUpdateBodyOptional(TypedDict, total=False):
    charge_interval_frequency: int
    cutoff_day_of_month: int
    cutoff_day_of_week: int
    discount_amount: str
    discount_type: ProductDiscountType
    expire_after_specific_number_of_charges: str
    modifiable_properties: list[str]
    order_day_of_month: int
    order_day_of_week: int
    order_interval_unit: ProductOrderIntervalUnit
    storefront_purchase_options: ProductStorefrontPurchaseOptions


class ProductUpdateBody(ProductCreateBodyOptional):
    shopify_product_id: int


class ProductDeleteBody(TypedDict):
    product_id: str


class ProductListQuery(TypedDict, total=False):
    id: str
    limit: str
    shopify_product_id: int
    page: str


class ProductResource(RechargeResource):
    """
    https://developer.rechargepayments.com/2021-01/products
    """

    object_list_key = "products"
    recharge_version: RechargeVersion = "2021-01"

    def create(self, body: ProductCreateBody):
        """Create a product.
        https://developer.rechargepayments.com/2021-01/products/products_create
        """
        required_scopes: list[RechargeScope] = ["write_products"]
        self.check_scopes(f"POST /{self.object_list_key}", required_scopes)

        return self._http_post(self.url, body)

    def get(self, product_id: str):
        """Get a product.
        https://developer.rechargepayments.com/2021-01/products/products_retrieve
        """
        required_scopes: list[RechargeScope] = ["read_products"]
        self.check_scopes(f"GET /{self.object_list_key}/:product_id", required_scopes)

        return self._http_get(f"{self.url}/{product_id}")

    def update(self, product_id: str, body: ProductUpdateBody):
        """Update a product.
        https://developer.rechargepayments.com/2021-01/products/products_update
        """
        required_scopes: list[RechargeScope] = ["write_products"]
        self.check_scopes(f"PUT /{self.object_list_key}/:product_id", required_scopes)

        return self._http_put(f"{self.url}/{product_id}", body)

    def delete(self, product_id: str):
        """Delete a product.
        https://developer.rechargepayments.com/2021-01/products/products_delete
        """
        required_scopes: list[RechargeScope] = ["write_products"]
        self.check_scopes(
            f"DELETE /{self.object_list_key}/:product_id", required_scopes
        )

        return self._http_delete(f"{self.url}/{product_id}")

    def list_(self, query: Optional[ProductListQuery] = None):
        """List products.
        https://developer.rechargepayments.com/2021-01/products/products_list
        """
        required_scopes: list[RechargeScope] = ["read_products"]
        self.check_scopes(f"GET /{self.object_list_key}", required_scopes)

        return self._http_get(self.url, query)

    def count(self):
        """Count products.
        https://developer.rechargepayments.com/2021-01/products/products_count
        """
        required_scopes: list[RechargeScope] = ["read_products"]
        self.check_scopes(f"GET /{self.object_list_key}/count", required_scopes)

        return self._http_get(f"{self.url}/count")

==> recharge_api-1.10.0-py3-none-any.whl: recharge/api/v1/products.py <==
from typing import TypedDict, Optional

from recharge.api import RechargeResource, RechargeScope, RechargeVersion
from recharge.exceptions import RechargeAPIError
from recharge.model.v1.product import (
    ProductDiscountType,
    ProductImages,
    ProductOrderIntervalUnit,
    ProductStorefrontPurchaseOptions,
    Product,
)


class ProductCreateBodyOptional(TypedDict, total=False):
    charge_interval_frequency: int
    cutoff_day_of_month: int
    cutoff_day_of_week: int
    discount_amount: str
    discount_type: ProductDiscountType
    expire_after_specific_number_of_charges: str
    modifiable_properties: list[str]
    order_day_of_month: int
    order_day_of_week: int
    order_interval_frequency_options: list[int]
    storefront_purchase_options: ProductStorefrontPurchaseOptions


class ProductCreateBody(ProductCreateBodyOptional):
    shopify_product_id: int


class ProductGetQuery(TypedDict, total=False):
    charge_interval_frequency: str
    created_at: str
    cutoff_day_of_month: str
    cutoff_day_of_week: str
    discount_amount: str
    discount_type: ProductDiscountType
    expire_after_specific_number_of_charges: str
    handle: str
    images: ProductImages
    modifiable_properties: list[str]
    number_charges_until_expiration: str
    order_day_of_month: str
    order_day_of_week: str
    order_interval_frequency: str
    order_interval_unit: ProductOrderIntervalUnit
    shopify_product_id: str
    storefront_purchase_options: ProductStorefrontPurchaseOptions
    title: str
    updated_at: str


class ProductUpdateBodyOptional(TypedDict, total=False):
    charge_interval_frequency: int
    cutoff_day_of_month: int
    cutoff_day_of_week: int
    discount_amount: str
    discount_type: ProductDiscountType
    expire_after_specific_number_of_charges: str
    modifiable_properties: list[str]
    order_day_of_month: int
    order_day_of_week: int
    order_interval_unit: ProductOrderIntervalUnit
    storefront_purchase_options: ProductStorefrontPurchaseOptions


class ProductUpdateBody(ProductCreateBodyOptional):
    shopify_product_id: int


class ProductDeleteBody(TypedDict):
    product_id: str


class ProductListQuery(TypedDict, total=False):
    id: str
    limit: str
    shopify_product_id: int
    page: str


class ProductResource(RechargeResource):
    """
    https://developer.rechargepayments.com/2021-01/products
    """

    object_list_key = "products"
    object_dict_key = "product"
    recharge_version: RechargeVersion = "2021-01"

    def create(self, body: ProductCreateBody) -> Product:
        """Create a product.
        https://developer.rechargepayments.com/2021-01/products/products_create
        """
        required_scopes: list[RechargeScope] = ["write_products"]
        self.check_scopes(f"POST /{self.object_list_key}", required_scopes)

        data = self._http_post(self.url, body)
        if not isinstance(data, dict):
            raise RechargeAPIError(f"Expected dict, got {type(data).__name__}")
        return Product(**data)

    def get(self, product_id: str) -> Product:
        """Get a product.
        https://developer.rechargepayments.com/2021-01/products/products_retrieve
        """
        required_scopes: list[RechargeScope] = ["read_products"]
        self.check_scopes(f"GET /{self.object_list_key}/:product_id", required_scopes)

        url = f"{self.url}/{product_id}"
        data = self._http_get(url)
        if not isinstance(data, dict):
            raise RechargeAPIError(f"Expected dict, got {type(data).__name__}")
        return Product(**data)

    def update(self, product_id: str, body: ProductUpdateBody) -> Product:
        """Update a product.
        https://developer.rechargepayments.com/2021-01/products/products_update
        """
        required_scopes: list[RechargeScope] = ["write_products"]
        self.check_scopes(f"PUT /{self.object_list_key}/:product_id", required_scopes)

        url = f"{self.url}/{product_id}"
        data = self._http_put(url, body)
        if not isinstance(data, dict):
            raise RechargeAPIError(f"Expected dict, got {type(data).__name__}")
        return Product(**data)

    def delete(self, product_id: str) -> dict:
        """Delete a product.
        https://developer.rechargepayments.com/2021-01/products/products_delete
        """
        required_scopes: list[RechargeScope] = ["write_products"]
        self.check_scopes(
            f"DELETE /{self.object_list_key}/:product_id", required_scopes
        )

        url = f"{self.url}/{product_id}"
        data = self._http_delete(url)
        if not isinstance(data, dict):
            raise RechargeAPIError(f"Expected dict, got {type(data).__name__}")
        return data

    def list_(self, query: Optional[ProductListQuery] = None) -> list[Product]:
        """List products.
        https://developer.rechargepayments.com/2021-01/products/products_list
        """
        required_scopes: list[RechargeScope] = ["read_products"]
        self.check_scopes(f"GET /{self.object_list_key}", required_scopes)

        data = self._http_get(self.url, query, expected=list)
        if not isinstance(data, list):
            raise RechargeAPIError(f"Expected list, got {type(data).__name__}")
        return [Product(**item) for item in data]

    def count(self) -> int:
        """Count products.
        https://developer.rechargepayments.com/2021-01/products/products_count
        """
        required_scopes: list[RechargeScope] = ["read_products"]
        self.check_scopes(f"GET /{self.object_list_key}/count", required_scopes)

        url = f"{self.url}/count"
        data = self._http_get(url)
        if not isinstance(data, dict):
            raise RechargeAPIError(f"Expected dict, got {type(data).__name__}")
        if "count" not in data:
            raise RechargeAPIError(f"Expected 'count' in response, got {data}")
        return data["count"]
